import java.awt.Color;

/** Class that crudely plots RN, k1*f and k2*f for experimentation
 *  with BigTheta.
 *  @author Josh Hug
 */

public class N2example {
    private static int MAX_N = 10;
    private static double k1 = 1;
    private static double k2 = 2;

    private interface func {
        double apply(double N);
    }

    /** Define f as N */
    private static class f implements func {
        public double apply(double N) {
            return N;
        }
    }


    /** Define f as N */
    private static class R implements func {
        public double apply(double N) {
            return (N * N * 2) + (N * 5) + 20;
        }
    }


    public static void main(String[] args) {
        drawCurves();
    }












    private static void drawCurves() {
        double[] nvals = new double[100];
        for (int i = 1; i < 100; i++) {
            nvals[i] = nvals[i-1] + MAX_N / 100.0;
        }

        func f = new f();
        func R = new R();

        double[] lowerBound = genScaledOutput(nvals, k1, f);
        double[] upperBound = genScaledOutput(nvals, k2, f);
        double[] RN = genOutput(nvals, R);

        double maxUpper = getMaxVal(upperBound);
        double maxR = getMaxVal(RN);

        StdDraw.setXscale(0, MAX_N);
        StdDraw.setYscale(0, Math.max(maxUpper, maxR));

        plot(nvals, lowerBound, StdDraw.RED);
        plot(nvals, upperBound, StdDraw.RED);
        plot(nvals, RN, StdDraw.BLACK);
    }


    private static double getMaxVal(double[] vals) {
        double maxVal = vals[0];
        for (int i = 0; i < vals.length; i++) {
            if (vals[i] > maxVal)
                maxVal = vals[i];
        }
        return maxVal;
    }


    private static double[] genScaledOutput(double[] input, double k, func Q) {
        double[] output = new double[input.length];
        for (int i = 0; i < input.length; i++) {
            output[i] = k * Q.apply(input[i]);
        }
        return output;
    }

    private static double[] genOutput(double[] input, func Q) {
        return genScaledOutput(input, 1, Q);
    }


    /** assumes x.length == y.length */
    private static void plot(double[] x, double[] y, Color c) {
        StdDraw.setPenColor(c);
        for (int i = 0; i < x.length - 1; i++) {
            StdDraw.line(x[i], y[i], x[i + 1], y[i + 1]);
        }
        StdDraw.show();
    }

} 